// Processing sketch to visualize color data from an Arduino/XIAO RP2040
// Expects serial data format: "R: <r_val> G: <g_val> B: <b_val>\n"
// where <r_val>, <g_val>, <b_val> are integers (expected 0-255 from Arduino)

import processing.serial.*;

Serial port;
String serialPortName = "COM46T"; // *** CHANGE THIS ***
int baudRate = 115200; // *** MUST MATCH ARDUINO BAUD RATE ***

String buff = ""; // Buffer for incoming serial data

// Variables to store parsed RGB values from serial
int Rval = 0;
int Gval = 0;
int Bval = 0;

// Global variable to store the current background color
color backgroundColor = color(0); // Default to black

void setup() {
  size(300, 300); // Window size
  println("Available serial ports:");
  println(Serial.list()); // Print list of available ports to console

  // --- IMPORTANT: Select the correct Serial Port ---
  // Find the port corresponding to your XIAO RP2040 in the list printed above
  // (e.g., "COM3", "/dev/ttyACM0", "/dev/cu.usbmodemXXXX")
  // and replace "YOUR_SERIAL_PORT" below.
  // If you know the index (e.g., it's the first one), you could use:
  // String[] portList = Serial.list();
  // if (portList.length > 0) {
  //  serialPortName = portList[0]; // Use the first available port
  // } else {
  //   println("Error: No serial ports found!");
  //   exit();
  // }
  // Or manually set it:
  serialPortName = "COM46"; // <-- *** REPLACE THIS WITH YOUR ACTUAL PORT ***

  println("Attempting to connect to: " + serialPortName + " at " + baudRate + " baud.");

  try {
    port = new Serial(this, serialPortName, baudRate);
    port.bufferUntil('\n'); // Read data until a newline character is received
    port.clear(); // Clear any initial garbage data
  } catch (RuntimeException e) {
    println("Error opening serial port '" + serialPortName + "'.");
    println("1. Check if the port name is correct.");
    println("2. Ensure the XIAO RP2040 is plugged in and the Arduino sketch is running.");
    println("3. Make sure no other program (like the Arduino Serial Monitor) is using the port.");
    e.printStackTrace();
    exit(); // Exit if the port can't be opened
  }
  println("Serial port opened successfully.");
}

void draw() {
  // Set the background using the updated color variable
  background(backgroundColor);

  // Display the current RGB values and Hex code on the screen
  fill(255 - red(backgroundColor), 255 - green(backgroundColor), 255 - blue(backgroundColor)); // Contrasting text color
  textSize(16);
  textAlign(LEFT, TOP);
  String displayText = "R: " + Rval + "\nG: " + Gval + "\nB: " + Bval + "\n#" + hex(backgroundColor, 6);
  text(displayText, 10, 10);
}

// This function is called automatically whenever data is received up to the bufferUntil character ('\n')
void serialEvent(Serial p) {
  buff = p.readStringUntil('\n'); // Read the complete line

  if (buff != null) {
    buff = trim(buff); // Remove leading/trailing whitespace & newline chars
    //println("Raw Received: [" + buff + "]"); // Uncomment for debugging

    try {
      // Find the positions of the R, G, B markers
      int rIndex = buff.indexOf("R:");
      int gIndex = buff.indexOf("G:");
      int bIndex = buff.indexOf("B:");

      // Ensure all markers are found
      if (rIndex != -1 && gIndex != -1 && bIndex != -1) {
        // Extract the values between the markers/end of string
        String rStr = buff.substring(rIndex + 2, gIndex).trim(); // +2 for "R:"
        String gStr = buff.substring(gIndex + 2, bIndex).trim(); // +2 for "G:"
        String bStr = buff.substring(bIndex + 2).trim();         // +2 for "B:" to end

        // Parse the integer values
        int tempR = Integer.parseInt(rStr);
        int tempG = Integer.parseInt(gStr);
        int tempB = Integer.parseInt(bStr);

        // Update the global RGB variables (constrain just in case)
        Rval = constrain(tempR, 0, 255);
        Gval = constrain(tempG, 0, 255);
        Bval = constrain(tempB, 0, 255);

        // Update the background color for the draw() loop
        backgroundColor = color(Rval, Gval, Bval);

        // Optional: Print parsed values to console for debugging
        // println("Parsed -> R:" + Rval + " G:" + Gval + " B:" + Bval);

      } else {
        println("Warning: Received malformed data (missing R/G/B markers): [" + buff + "]");
      }
    } catch (Exception e) {
      println("Error parsing serial data: [" + buff + "]");
      e.printStackTrace(); // Print detailed error
    }
  }
}
