#include <Wire.h>             // Include the I2C library
#include "Adafruit_APDS9960.h" // Include the APDS9960 library

// Create an instance of the sensor object
Adafruit_APDS9960 apds;

// Variables to store raw sensor readings (range 0-65535 typically)
uint16_t r_raw, g_raw, b_raw, c_raw;

// Variables to store scaled color values (target range 0-255)
// These are the values we will send to Processing
uint8_t r_scaled, g_scaled, b_scaled;

// Define the Baud Rate (must match Processing sketch)
#define SERIAL_BAUD 115200

// --- SETUP: Runs once when the board powers on or resets ---
void setup() {
  // Start Serial communication at the defined baud rate
  Serial.begin(SERIAL_BAUD);
  // Optional delay allows time to open Serial Monitor for debugging, if needed
  // Can be removed or shortened for faster startup.
  delay(1000);
  Serial.println("XIAO RP2040 + APDS9960 Color Sensor");
  Serial.println("Sending data to Processing sketch...");
  Serial.println("------------------------------------");

  // Initialize I2C communication (for the sensor)
  // Default pins for XIAO RP2040 (SDA=Pin6/D4, SCL=Pin7/D5) usually work
  Wire.begin();

  // Initialize the APDS9960 sensor
  if (!apds.begin()) {
    // If the sensor is not found, print an error and stop execution
    Serial.println("ERROR: Failed to initialize APDS9960 sensor!");
    Serial.println("Check wiring: VIN(3V3), GND, SDA, SCL");
    while (1) delay(10); // Infinite loop halts the program
  }
  Serial.println("APDS9960 sensor initialized successfully.");

  // Enable the color sensing engine on the sensor
  apds.enableColor(true);

  // Optional: Adjust sensor gain and integration time if needed
  // Default: 4x gain, 2.78ms integration time
  // apds.setADC_Gain(APDS9960_AGAIN_1X); // Example: Set gain to 1x
  // apds.setADCTime(10); // Example: Set integration time to ~27.8ms
}

// --- LOOP: Runs repeatedly after setup() completes ---
void loop() {
  // 1. Wait until the sensor has color data available
  while (!apds.colorDataReady()) {
    delay(5); // Small delay to prevent the loop from running too fast
  }

  // 2. Read the raw color data from the sensor
  // Values are read into the uint16_t variables (0-65535)
  apds.getColorData(&r_raw, &g_raw, &b_raw, &c_raw);

  // 3. Scale the raw R, G, B values to the 0-255 range
  // We use the 'clear' value (c_raw) to normalize for brightness.
  // This helps make the displayed color less dependent on overall light level.
  if (c_raw == 0) {
    // Avoid division by zero if sensor reads no light (total darkness)
    r_scaled = 0;
    g_scaled = 0;
    b_scaled = 0;
  } else {
    // Calculate the scaled value: (raw_color / clear_light) * 255
    // Use 'long' casting during multiplication to prevent intermediate overflow.
    // Use 'constrain' to ensure the final value is strictly within 0-255.
    r_scaled = (uint8_t)constrain(((long)r_raw * 255L) / c_raw, 0, 255);
    g_scaled = (uint8_t)constrain(((long)g_raw * 255L) / c_raw, 0, 255);
    b_scaled = (uint8_t)constrain(((long)b_raw * 255L) / c_raw, 0, 255);
  }

  // 4. Send the SCALED data over Serial in the specific format Processing expects
  // Format: "R: <value> G: <value> B: <value>\n"
  Serial.print("R: "); Serial.print(r_scaled); // Send the scaled red value (0-255)
  Serial.print(" G: "); Serial.print(g_scaled); // Send the scaled green value (0-255)
  Serial.print(" B: "); Serial.print(b_scaled); // Send the scaled blue value (0-255)
  Serial.println();                         // Send the newline character to finish the message

  // 5. Wait a short period before the next reading
  delay(100); // Update about 10 times per second. Adjust as needed.
}